@@ -1589,6 +1589,126 @@ impl<T: Noned + Copy> Optioned<T> {
         }
     }
 
+    /// Returns this option if it contains a value, otherwise returns the other.
+    ///
+    /// Arguments passed to `or` are eagerly evaluated;
+    /// if you are passing the result of a function call,
+    /// it is recommended to use `or_else`, which is lazily evaluated.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use optional::{Optioned, some, none};
+    /// let x = some(2);
+    /// let y = none();
+    /// assert_eq!(x.or(y), some(2));
+    ///
+    /// let x = none();
+    /// let y = some(100);
+    /// assert_eq!(x.or(y), some(100));
+    ///
+    /// let x = some(2);
+    /// let y = some(100);
+    /// assert_eq!(x.or(y), some(2));
+    ///
+    /// let x: Optioned<u32> = none();
+    /// let y = none();
+    /// assert_eq!(x.or(y), none());
+    /// ```
+    #[inline]
+    pub fn or(self, other: Optioned<T>) -> Optioned<T> {
+        if self.is_some() {
+            self
+        } else {
+            other
+        }
+    }
+
+    /// Returns this option if it contains a value, otherwise calls `f` and returns the result.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use optional::{Optioned, some, none};
+    /// fn nothing() -> Optioned<u32> { none() }
+    /// fn something() -> Optioned<u32> { some(1) }
+    ///
+    /// assert_eq!(some(2).or_else(something), some(2));
+    /// assert_eq!(none().or_else(something), some(1));
+    /// assert_eq!(none().or_else(nothing), none());
+    /// ```
+    pub fn or_else<F>(self, f: F) -> Optioned<T>
+    where
+        F: FnOnce() -> Optioned<T>,
+    {
+        if self.is_some() {
+            self
+        } else {
+            f()
+        }
+    }
+
+    /// Returns the `None` value for type `U` if this value or `other` contains their respective
+    /// `None` values. Otherwise returns the `other` `Optioned` struct. 
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use optional::{Optioned, some, none};
+    /// let the_other = some::<u32>(42);
+    ///
+    /// assert_eq!(some('a').and(the_other), some(42));
+    /// assert_eq!(none::<char>().and(the_other), none::<u32>());
+    /// assert_eq!(some('a').and(none::<u32>()), none::<u32>());
+    /// assert_eq!(none::<char>().and(none::<u32>()), none::<u32>());
+    /// ```
+    #[inline]
+    pub fn and<U>(self, other: Optioned<U>) -> Optioned<U>
+    where
+        U: Noned + Copy
+    {
+        if self.is_some() {
+           other
+        } else {
+            none::<U>()
+        }
+    }
+
+    /// Returns this `Optioned` if it contains the the `None` value, otherwise calls `f` with
+    /// the contained value and returns the result as an `Optioned<U>`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use optional::{Optioned, some, none, wrap};
+    /// fn nothing() -> Optioned<u32> { none() }
+    /// fn something() -> Optioned<u32> { some(1) }
+    /// fn add_two(val: u32) -> Optioned<u32> {
+    ///   wrap( val + 2)
+    /// }
+    /// 
+    /// fn failed_function(val: u32) -> Optioned<u32> {
+    ///   none()
+    /// }
+    ///
+    /// assert_eq!(some(2).and_then(add_two), some(4));
+    /// assert_eq!(none().and_then(add_two), none());
+    /// assert_eq!(some(2).and_then(failed_function), none());
+    /// assert_eq!(none().and_then(failed_function), none());
+    /// ```
+    #[inline]
+    pub fn and_then<F,U>(self, f: F) -> Optioned<U>
+    where
+        F: FnOnce(T) -> Optioned<U>,
+        U: Noned + Copy
+    {
+        if self.is_some() {
+            f(self.value)
+        } else {
+            none()
+        }
+    }
+
     /// Takes the value out of the `Optioned` and returns ist as
     /// `Option<T>`, changing self to `None`.
     ///
@@ -1734,3 +1854,10 @@ impl<T: Noned + Copy> Into<Option<T>> for Optioned<T> {
         self.as_option()
     }
 }
+
+impl<T: Noned + Copy> From<T> for Optioned<T> {
+    #[inline]
+    fn from(o: T) -> Optioned<T> {
+        wrap(o)
+    }
+}
